

var styleNode = document.createElement('style');
styleNode.innerHTML = " .ixbrl-inspector { position: fixed; right: -300px; top: 0px; bottom: 0px; width: 300px; /*height: 400px;*/ border-left: solid 1px black; box-shadow: -5px 0px 5px #777777; background-color: #bbbbff; opacity: 0.9; z-index: 10; box-sizing: border-box; font-family: sans-serif; font-size: initial; } .ixbrl-inspector h1 { box-sizing: content-box; padding: 10px; color: white; background-color: #222277; font-size: 20px; margin: 0 0; display: block; font-family: sans-serif; text-align: left; position: absolute; transform: rotate(270deg); width: 150px; top: 63px; left: -108px; border-left: solid 1px black; border-top: solid 1px black; cursor: pointer; text-transform: none; } .ixbrl-inspector > div { padding: 10px; } .ixbrl-highlight, .ixbrl-selected { background-color: yellow; } .ixbrl-selected, .ixbrl-element:hover { outline: dashed 1px black; } .ixbrl-element { cursor: pointer; position: relative; } .ixbrl-fact-details { padding: 10px; } .ixbrl-fact-detail-title { font-weight: bold; margin-bottom: 10px; margin-top: 15px; border-top: dashed 1px #aaa; padding-top: 15px; } .ixbrl-fact-detail-value { padding-left: 10px; margin-bottom: 10px; }";
document.head.appendChild(styleNode);

var contexts = {};
var units = {};

var inspectorPos = -300;
var inspectorDragged = false;

function processiXBRL(n) {
  // XXX ignore prefixes

  if(n.nodeType == 1 && (n.nodeName == 'IX:NONNUMERIC' || n.nodeName == 'IX:NONFRACTION')) {
    var wrapper = "span";
    nn = n.getElementsByTagName("*");
    for (i = 0; i < nn.length; i++) {
      if($(nn[i]).css("display") === "block") {
        wrapper = 'div';
        break;
      }
    }
    if(n.nodeName == 'IX:NONFRACTION') {
      $(n).wrap('<' + wrapper + ' class="ixbrl-element ixbrl-element-nonfraction"></' + wrapper + '>');
    }
    if(n.nodeName == 'IX:NONNUMERIC') {
      $(n).wrap('<' + wrapper + ' class="ixbrl-element ixbrl-element-nonnumeric"></' + wrapper + '>');
    }
  }
  else if(n.nodeType == 1 && n.nodeName == 'XBRLI:CONTEXT') {
    var context = {};
    var p = n.getElementsByTagName("XBRLI:PERIOD")[0];
    if(p.getElementsByTagName("XBRLI:STARTDATE").length > 0) {
      context.startDate = p.getElementsByTagName("XBRLI:STARTDATE")[0].textContent
      context.endDate = p.getElementsByTagName("XBRLI:ENDDATE")[0].textContent
    }
    else {
      context.instant = p.getElementsByTagName("XBRLI:INSTANT")[0].textContent
    }
    var segment = n.getElementsByTagName("XBRLI:ENTITY")[0].getElementsByTagName("XBRLI:SEGMENT")[0];
    context.dimensions = {};
    if(segment) {
      var explicitMembers = segment.getElementsByTagName("XBRLDI:EXPLICITMEMBER");
      for(var i=0; i < explicitMembers.length; i++) {
        context.dimensions[explicitMembers[i].getAttribute("dimension")] = explicitMembers[i].textContent;
      }
    }
    var identifier = n.getElementsByTagName("XBRLI:ENTITY")[0].getElementsByTagName("XBRLI:IDENTIFIER")[0];
    context.entity = {
      scheme: identifier.getAttribute("scheme"),
      identifier: identifier.textContent
    }
    
    contexts[n.getAttribute("id")] = context;
  }
  else if(n.nodeType == 1 && n.nodeName == 'XBRLI:UNIT') {
    var u = n.getElementsByTagName("XBRLI:MEASURE")[0].textContent;
    units[n.getAttribute("id")] = u;
  }
  for (var i=0; i < n.childNodes.length; i++) {
    processiXBRL(n.childNodes[i]);
  }

}

function ixbrlDetails(x) {
  n = x.get(0).childNodes[0];
  var details = {};
  details.concept = n.getAttribute("name");
  details.rawValue = n.textContent;
  if(x.hasClass("ixbrl-element-nonnumeric")) {
    details.value = details.rawValue;
  }
  else {
    details.scale = n.getAttribute("scale") || 0;
      var format = n.getAttribute("format") || "";
      if (!format.includes("dot")) {
          details.value = details.rawValue.replace(/[^0-9,]/g, '').replace(/[,]/g, '.') * Math.pow(10, details.scale);
      }
      else {
          details.value = details.rawValue.replace(/[^0-9.]/g, '') * Math.pow(10, details.scale);
      }
    if (n.getAttribute("sign")) {
      details.value *= -1;
    }
    details.value = details.value.toFixed(2).replace(/(\d)(?=(\d{3})+\.)/g, '$1,');
  }
  details.contextId = n.getAttribute("contextRef");
  details.unitId = n.getAttribute("unitRef");
  return details;
}

function symbolForUnit(unit) {
  var s = '';
  if(unit && unit.startsWith('iso4217:')) {
    matches = unit.match(/:([^:]+)$/);
    switch(matches[1]) {
      case "EUR":
        s = '\u20ac';
        break;
      case "GBP":
        s = '\u00a3';
        break;
      case "USD":
        s = '$';
        break;
    }
  }
  return s;
}

//$('head').append('<link rel="stylesheet" href="ixbrl-inspector.css" type="text/css" />');
var wrapper = document.createElement("div");
wrapper.innerHTML = '\
    <div class="ixbrl-inspector">\
      <h1>Inline XBRL</h1>\
\
      <div>\
        <form>\
        <p>\
        <label><input type="checkbox" id="ixbrl-show-all-tags"> Highlight all tags</label>\
        </p>\
      </form>\
      </div>\
      <div class="ixbrl-fact-details">\
        <div class="ixbrl-fact-detail-title">\
          Line item\
        </div>\
        <div class="ixbrl-fact-detail-value" id="ixbrl-fact-concept-name">\
          -\
        </div>\
        <div class="ixbrl-fact-detail-title">\
          Value\
        </div>\
        <div class="ixbrl-fact-detail-value" id="ixbrl-fact-value">\
          -\
        </div>\
        <div class="ixbrl-fact-detail-title">\
          Period\
        </div>\
        <div class="ixbrl-fact-detail-value" id="ixbrl-fact-period">\
          -\
        </div>\
        <div id="ixbrl-fact-dimensions"></div>\
        <div class="ixbrl-fact-detail-title">\
          Units\
        </div>\
        <div class="ixbrl-fact-detail-value" id="ixbrl-fact-units">\
          -\
        </div>\
        <div class="ixbrl-fact-detail-title">\
          Entity\
        </div>\
        <div class="ixbrl-fact-detail-value" id="ixbrl-fact-entity">\
          -\
        </div>\
      </div>\
    </div>\
';

function inspectorHidden() {
  return inspectorPos < 0;
}

function showHideInspector() {
    var i = $(".ixbrl-inspector");
    if(inspectorPos < 0) {
      inspectorPos = 0;
    }
    else {
      inspectorPos = 0 - i.width();
    }
    i.animate({"right": inspectorPos});
}


$(function() {
  document.body.appendChild(wrapper);
  processiXBRL(document.body);
  interact('.ixbrl-inspector', { allowFrom: ".ixbrl-inspector h1" })
    .resizable({
      preserveAspectRatio: false,
      edges: { left: true, right: false, bottom: false, top: false }
    })
    .on('down', function (event) {
      inspectorDragged = false;
    })
    .on('up', function (event) {
      if(!inspectorDragged) { 
        showHideInspector();
        $('html').css('cursor', '');
      }
    })
    .on('resizemove', function (event) {
      var target = event.target;

      var dx = event.deltaRect.left
      var w = $(target).width()
      w -= dx;
      if(inspectorPos < 0 && w > 200) {
        w += inspectorPos;
        inspectorPos = 0;
      }
      if(w < 200) {
        inspectorPos -= (200 - w);
        w = 200;
      }
      $(target).width(w);
      $(target).css("right", inspectorPos + "px");
      inspectorDragged = true;
  });

  $("body").click(function (e) {
    if($(e.target).closest(".ixbrl-element").length == 0) {
      // TODO Clear inspector.
      $(".ixbrl-element").removeClass("ixbrl-selected");
    }
  });

  $(".ixbrl-element").click(function (e) {
    if(inspectorHidden()) {
      showHideInspector();
    }
    /* Only highlight the innermost element */
    e.stopPropagation();
    $(".ixbrl-element").removeClass("ixbrl-selected");
    $(this).addClass("ixbrl-selected");
    details = ixbrlDetails($(this));

    $("#ixbrl-fact-concept-name").text(details.concept);

    var u = units[details.unitId];
    $("#ixbrl-fact-value").text(symbolForUnit(u) + " " + details.value);
    var c = contexts[details.contextId];
    if(typeof c.startDate === "undefined") {
      $("#ixbrl-fact-period").text(c.instant);
    } else {
      $("#ixbrl-fact-period").text(c.startDate + " to " + c.endDate);
    }

    if(u) {
      $("#ixbrl-fact-units").text(u);
    }
    else {
      $("#ixbrl-fact-units").text("-");
    }

    var dimStr = '';
    for(var d in c.dimensions) {
      dimStr += '<div class="ixbrl-fact-detail-title">' + d + '</div><div class="ixbrl-fact-detail-value">' + c.dimensions[d] + '</div>';
    }
    $("#ixbrl-fact-dimensions").html(dimStr);

    var entityStr;
    if(c.entity.scheme == "http://standards.iso.org/iso/17442") {
      entityStr = '<a target="_blank" href="https://www.gleif.org/lei/' + c.entity.identifier + '">' + c.entity.identifier + '</a>';
    }
    else if(c.entity.scheme == "http://www.companieshouse.gov.uk/") {
      entityStr = '<a target="_blank" href="https://beta.companieshouse.gov.uk/company/' + c.entity.identifier + '">' + c.entity.identifier + '</a>';
    }
    else {
      entityStr = c.entity.identifier;
    }
    $("#ixbrl-fact-entity").html(entityStr);

  });

  $('#ixbrl-show-all-tags').change(function(e){
    if(this.checked) {
      $(".ixbrl-element").addClass("ixbrl-highlight");
    }
    else {
      $(".ixbrl-element").removeClass("ixbrl-highlight");
    }
  });
});

